@@ -26,8 +26,10 @@
 #include "Ident.hpp"
 #ifdef __BSD  // A bodge to get round the problem of a truncated UDS file
 	#define __IPC "/tmp/.dguardianipcc"
+	#define __URLIPC "/tmp/.dguardianurlipcc"
 #else
 	#define __IPC "/tmp/.dguardianipc"
+	#define __URLIPC "/tmp/.dguardianurlipc"
 #endif
 #include "NaughtyFilter.hpp"
 #include "HTTPHeader.hpp"
@@ -37,7 +39,6 @@
 #include <arpa/inet.h>
 #include <cstdio>
 #include <algorithm>
-#include <istream.h>
 #include <fstream>
 #include <iostream>
 #include <netdb.h>
@@ -45,10 +46,20 @@
 #include <unistd.h>
 #include <sys/time.h>
 
+#ifdef __GCCVER3
+    #include <istream>
+#else
+    #include <istream.h>
+#endif
+
 extern OptionContainer o;
 
 void ConnectionHandler::handleConnection(Socket peerconn) {
 
+    struct timeval thestart;
+    struct timezone notused;
+    gettimeofday(&thestart, &notused);
+
     peerconn.setTimeout(10);
 
     HTTPHeader header;  // to hold the incoming client request header
@@ -67,6 +78,10 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
     bool wasrequested = false;
     bool isexception = false;
     bool isourwebserver = false;
+    bool wasclean = true;
+    bool cachehit = false;
+
+    std::string mimetype = "-";
 
     String url;
     String urld;
@@ -84,8 +99,6 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
         std::cout << clientip << std::endl;
     #endif
 
-
-
     Socket proxysock;  // to hold connection to proxy
 
     // connect to proxy
@@ -104,12 +117,19 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
         header.in(peerconn);  // get header from client
         url = header.url();
         urld = header.decode(url);
+
         if (header.malformedURL(url)) {
             // checks for bad URLs to prevent security hole
             try { // writestring throws exception on error/timeout
                 peerconn.writeString("HTTP/1.0 400 Bad Request\n");
                 peerconn.writeString("Content-Type: text/html\n\n");
-                peerconn.writeString("<HTML><HEAD><TITLE>DansGuardian - 400 Bad Request</TITLE></HEAD><BODY><H1>DansGuardian - 400 Bad Request</H1> The requested URL is malformed.</BODY></HTML>\n");
+                peerconn.writeString("<HTML><HEAD><TITLE>DansGuardian - 400 Bad Request</TITLE></HEAD><BODY><H1>DansGuardian - 400 Bad Request</H1> ");
+                peerconn.writeString(o.language_list.getTranslation(200));
+                // The requested URL is malformed.
+                peerconn.writeString("</BODY></HTML>\n");
+            } catch (exception& e) {}
+            try {
+                proxysock.close();  // close connection to proxy
             } catch (exception& e) {}
             return;
         }
@@ -117,8 +137,6 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
         std::string clientuser = ident.getUsername(&header, &peerconn);
                                                    // extract username
 
-        // Modification based on a submitted patch by
-        // Jimmy Myrick (jmyrick@tiger1.tiger.org)
         if (o.use_xforwardedfor == 1) {
             std::string xforwardip = header.getXForwardedForIP();
             if (xforwardip.length() > 6) {
@@ -128,19 +146,19 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
                 std::cout << "using x-forwardedfor:" << clientip << std::endl;
             #endif
         }
-
-
         if (o.forwarded_for == 1) {
             header.addXForwardedFor(clientip);  // add squid-like entry
         }
 
         if (o.inipexceptions(clientip)) {  // admin pc
             isexception = true;
-            exceptionreason = "Exception client IP match.";
+            exceptionreason = o.language_list.getTranslation(600);
+            // Exception client IP match.
         }
         else if (o.inuserexceptions(clientuser)) { // admin user
             isexception = true;
-            exceptionreason = "Exception client user match.";
+            exceptionreason = o.language_list.getTranslation(601);
+            // Exception client user match.
         }
         else if (o.inexceptions(urld)) {  // allowed site
             if (o.iswebserver(url)) {
@@ -148,18 +166,21 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
             }
             else {
                 isexception = true;
-                exceptionreason = "Exception site match.";
+                exceptionreason = o.language_list.getTranslation(602);
+                // Exception site match.
             }
         }
         else if (o.inurlexceptions(urld)) {  // allowed url
             isexception = true;
-            exceptionreason = "Exception url match.";
+            exceptionreason = o.language_list.getTranslation(603);
+            // Exception url match.
         }
 
         #ifdef DGDEBUG
             std::cout << "extracted url:" << urld << std::endl;
         #endif
 
+
         if ( ((o.inipexceptions(clientip)  // admin pc
                 || o.inuserexceptions(clientuser) // admin user
                 || o.inurlexceptions(urld) // ok part of site
@@ -177,8 +198,9 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
                 fdt.tunnel(proxysock.getFD(), peerconn.getFD()); // not expected to exception
                 docsize = fdt.throughput;
                 if (!isourwebserver) {  // don't log requests to the web server
-                    decideHowToLog(clientuser, clientip, url.toCharArray(), exceptionreason, header.requesttype().toCharArray(), docsize, o.ll, false, isexception, o.log_exception_hits, false);
+                    decideHowToLog(clientuser, clientip, url.toCharArray(), exceptionreason, header.requesttype().toCharArray(), docsize, o.ll, false, isexception, o.log_exception_hits, false, &thestart, cachehit, 200, mimetype);
                 }
+
             } catch (exception& e) {}
             try {
                 proxysock.close();  // close connection to proxy
@@ -198,22 +220,29 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
 
             if (o.blanketblock == 1) {
                 checkme.isItNaughty = true;
-                checkme.whatIsNaughty = "Blanket Block is active and that site is not on the white list.";
+                checkme.whatIsNaughty = o.language_list.getTranslation(502);
+                // Blanket Block is active and that site is not on the white list.
                 checkme.whatIsNaughtyLog = checkme.whatIsNaughty;
             }
 
             if (!checkme.isItNaughty) {
                 if (o.inBannedIPList(clientip)) {
                     checkme.isItNaughty = true;
-                    checkme.whatIsNaughtyLog = "Your IP address is not allowed to web browse: " + clientip;
-                    checkme.whatIsNaughty = "Your IP address is not allowed to web browse.";
+                    checkme.whatIsNaughtyLog = o.language_list.getTranslation(100);
+                    // Your IP address is not allowed to web browse:
+                    checkme.whatIsNaughtyLog += clientip;
+
+                    checkme.whatIsNaughty = o.language_list.getTranslation(101);
+                    // Your IP address is not allowed to web browse.
                 }
             }
 
             if (!checkme.isItNaughty) {
                 if (o.inBannedUserList(clientuser)) {
                     checkme.isItNaughty = true;
-                    checkme.whatIsNaughtyLog = "Your username is not allowed to web browse: " + clientuser;
+                    checkme.whatIsNaughtyLog = o.language_list.getTranslation(102);
+                    // Your username is not allowed to web browse:
+                    checkme.whatIsNaughtyLog += clientuser;
                     checkme.whatIsNaughty = checkme.whatIsNaughtyLog;
                 }
             }
@@ -222,14 +251,19 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
                 i = o.inBannedRegExpURLList(temp);
                 if (i >= 0) {
                     checkme.isItNaughty = true;
-                    checkme.whatIsNaughtyLog = "Banned Regular Expression URL: " + o.banned_regexpurl_list.getItemAt(i);
-                    checkme.whatIsNaughty = "Banned Regular Expression URL found.";
+                    checkme.whatIsNaughtyLog = o.language_list.getTranslation(503);
+                    // Banned Regular Expression URL:
+                    checkme.whatIsNaughtyLog += o.banned_regexpurl_list.getItemAt(i);
+                    checkme.whatIsNaughty = o.language_list.getTranslation(504);
+                    // Banned Regular Expression URL found.
                 }
             }
             if (!checkme.isItNaughty) {
                 i = o.inBannedURLList(temp);
                 if (i >= 0) {
-                    checkme.whatIsNaughty = "Banned URL: " + o.banned_url_list.getItemAt(i);
+                    checkme.whatIsNaughty = o.language_list.getTranslation(501);
+                    // Banned URL:
+                    checkme.whatIsNaughty += o.banned_url_list.getItemAt(i);
                     checkme.whatIsNaughtyLog = checkme.whatIsNaughty;
                     checkme.isItNaughty = true;
                 }
@@ -238,11 +272,13 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
                 i = o.inBannedSiteList(temp);
                 if (i == -2) {
                     checkme.isItNaughty = true;
-                    checkme.whatIsNaughty = "Blanket IP Block is active and that address is an IP only address.";
+                    checkme.whatIsNaughty = o.language_list.getTranslation(502);
+                    //Blanket IP Block is active and that address is an IP only address.
                     checkme.whatIsNaughtyLog = checkme.whatIsNaughty;
                 }
                 if (i >= 0) {
-                    checkme.whatIsNaughty = "Banned site: " + o.banned_site_list.getItemAt(i);
+                    checkme.whatIsNaughty = o.language_list.getTranslation(500); // banned site
+                    checkme.whatIsNaughty += o.banned_site_list.getItemAt(i);
                     checkme.whatIsNaughtyLog = checkme.whatIsNaughty;
                     checkme.isItNaughty = true;
                 }
@@ -255,19 +291,20 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
                         std::cout << "is post upload" << std::endl;
                     #endif
                     if (o.max_upload_size == 0) {
-                        checkme.whatIsNaughty = "Web upload is banned.";
+                        checkme.whatIsNaughty = o.language_list.getTranslation(700);
+                        // Web upload is banned.
                         checkme.whatIsNaughtyLog = checkme.whatIsNaughty;
                         checkme.isItNaughty = true;
                     }
                     else if (header.contentlength() > o.max_upload_size) {
-                        checkme.whatIsNaughty = "Web upload limit exceeded.";
+                        checkme.whatIsNaughty = o.language_list.getTranslation(701);
+                        // Web upload limit exceeded.
                         checkme.whatIsNaughtyLog = checkme.whatIsNaughty;
                         checkme.isItNaughty = true;
                     }
                 }
             }
 
-
             if (!checkme.isItNaughty && header.requesttype().startsWith("CONNECT")) {
                 // can't filter content of CONNECT
                 proxysock.readyForOutput(10);  // exception on timeout or error
@@ -277,7 +314,7 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
                     // tunnel from client to proxy and back
                     fdt.tunnel(proxysock.getFD(), peerconn.getFD()); // not expected to exception
                     docsize = fdt.throughput;
-                    decideHowToLog(clientuser, clientip, url.toCharArray(), exceptionreason, header.requesttype().toCharArray(), docsize, o.ll, false, isexception, o.log_exception_hits, false);
+                    decideHowToLog(clientuser, clientip, url.toCharArray(), exceptionreason, header.requesttype().toCharArray(), docsize, o.ll, false, isexception, o.log_exception_hits, false, &thestart, cachehit, 200, mimetype);
                 } catch (exception& e) {}
                 try {
                     proxysock.close();  // close connection to proxy
@@ -296,17 +333,20 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
                 #endif
                 wasrequested = true;  // so we know where we are later
 
-                temp = docheader.getcontenttype();
+                mimetype = docheader.getcontenttype().toCharArray();
 
-                i = o.banned_mimetype_list.findInList(temp.toCharArray());
+                i = o.banned_mimetype_list.findInList((char*)mimetype.c_str());
                 if (i >= 0) {
-                    checkme.whatIsNaughty = "Banned MIME Type: " + o.banned_mimetype_list.getItemAt(i);
+                    checkme.whatIsNaughty = o.language_list.getTranslation(800);
+                    // Banned MIME Type:
+                    checkme.whatIsNaughty += o.banned_mimetype_list.getItemAt(i);
+
                     checkme.whatIsNaughtyLog = checkme.whatIsNaughty;
                     checkme.isItNaughty = true;
                 }
                 #ifdef DGDEBUG
-                    std::cout << temp.length() << std::endl;
-                    std::cout << ":" << temp;
+                    std::cout << mimetype.length() << std::endl;
+                    std::cout << ":" << mimetype;
                     std::cout << ":" << std::endl;
                 #endif
 
@@ -325,7 +365,9 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
                         tempdispos = "http://foo.bar/" + tempdispos;
                         i = o.inBannedExtensionList(tempdispos);
                         if (i >= 0) {
-                            checkme.whatIsNaughty = "Banned extension: " + o.banned_extension_list.getItemAt(i);
+                            checkme.whatIsNaughty = o.language_list.getTranslation(900);
+                            // Banned extension:
+                            checkme.whatIsNaughty += o.banned_extension_list.getItemAt(i);
                             checkme.whatIsNaughtyLog = checkme.whatIsNaughty;
                             checkme.isItNaughty = true;
                         }
@@ -334,19 +376,23 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
                         if (!tempurl.contains("?")) {
                             i = o.inBannedExtensionList(tempurl);
                             if (i >= 0) {
-                                checkme.whatIsNaughty = "Banned extension: " + o.banned_extension_list.getItemAt(i);
+                                checkme.whatIsNaughty =  o.language_list.getTranslation(900);
+                                // Banned extension:
+                                checkme.whatIsNaughty += o.banned_extension_list.getItemAt(i);
                                 checkme.whatIsNaughtyLog = checkme.whatIsNaughty;
                                 checkme.isItNaughty = true;
                             }
                         }
-                        if (temp.contains("application/")) {
+                        if (String(mimetype.c_str()).contains("application/")) {
                             while (tempurl.endsWith("?")) {
                                 tempurl.chop();
                             }
                             while(tempurl.contains("/")) {  // no slash no url
                                 i = o.inBannedExtensionList(tempurl);
                                 if (i >= 0) {
-                                    checkme.whatIsNaughty = "Banned extension: " + o.banned_extension_list.getItemAt(i);
+                                    checkme.whatIsNaughty =  o.language_list.getTranslation(900);
+                                    // Banned extension:
+                                    checkme.whatIsNaughty += o.banned_extension_list.getItemAt(i);
                                     checkme.whatIsNaughtyLog = checkme.whatIsNaughty;
                                     checkme.isItNaughty = true;
                                     break;
@@ -361,8 +407,12 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
                     }
                 }
 
-
                 if (docheader.iscontenttype("text") && !checkme.isItNaughty) {
+                    // here we check if its a known good one
+                    // if so we skip content checking
+
+
+
                     waschecked = true;
                     proxysock.checkForInput(60);
                     if (docheader.isCompressed()) {
@@ -378,6 +428,7 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
                     #ifdef DGDEBUG
                         std::cout << "got body from proxy" << std::endl;
                     #endif
+
                     int dblen = docbody.length();
                     docsize = dblen;
 
@@ -386,15 +437,55 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
                         // need to modify header to mark as not compressed
                         // it also modifies Content-Length as well
                     }
+
+
+                    wasclean = false;
+                    if (o.url_cache_number > 0) {
+                        if (wasClean(urld)) {
+                            wasclean = true;
+                            #ifdef DGDEBUG
+                                std::cout << "url was clean skipping content checking" << std::endl;
+                            #endif
+                        }
+                    }
+                    if (!wasclean) {
+                        #ifdef DGDEBUG
+                            system("date");
+                        #endif
+                        if (dblen < o.max_content_filter_size
+                            || o.max_content_filter_size == 0) {
+                            checkme.checkme(&docbody);  // content filtering
+                        }
+                        #ifdef DGDEBUG
+                            else {
+                                std::cout << "content length large so skipping content filtering" << std::endl;
+                            }
+                            system("date");
+                        #endif
+                    }
+                    else {
+                        cachehit = true;
+                    }
                     #ifdef DGDEBUG
                         system("date");
                     #endif
-                    checkme.checkme(&docbody);  // content filtering
+                    if (dblen < o.max_content_filter_size
+                        || o.max_content_filter_size == 0) {
+                        docbody.contentRegExp();  // content modifying
+                                              // uses global variable
+                    }
                     #ifdef DGDEBUG
+                        else {
+                            std::cout << "content length large so skipping content modifying" << std::endl;
+                        }
                         system("date");
                     #endif
 
+                    docheader.setContentLength(docbody.buffer_length);
+                // here if its OK then request addition to good url database
+
                 }
+
             }
 
             if (checkme.isException) {
@@ -406,9 +497,15 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
                 checkme.isItNaughty = false;
             }
 
+            if (o.url_cache_number > 0) {
+                if (!wasclean && !checkme.isItNaughty) {
+                    addToClean(urld);
+                }
+            }
+
             if (checkme.isItNaughty) {  // then we deny
 
-                decideHowToLog(clientuser, clientip, url.toCharArray(), checkme.whatIsNaughtyLog, header.requesttype().toCharArray(), docsize, o.ll, true, false, false, false);
+                decideHowToLog(clientuser, clientip, url.toCharArray(), checkme.whatIsNaughtyLog, header.requesttype().toCharArray(), docsize, o.ll, true, false, false, false, &thestart, cachehit, 403, mimetype);
 
                 try { // writestring throws exception on error/timeout
                     if (o.reporting_level == 3) {
@@ -445,17 +542,17 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
                     else if (o.reporting_level > 0) {
                         proxysock.close();  // finshed with proxy
                         peerconn.readyForOutput(10);
-                        if (checkme.whatIsNaughty.length() > 256) {
-                            checkme.whatIsNaughty = String(checkme.whatIsNaughty.c_str()).subString(0, 512).toCharArray();
+                        if (checkme.whatIsNaughty.length() > 2048) {
+                            checkme.whatIsNaughty = String(checkme.whatIsNaughty.c_str()).subString(0, 2048).toCharArray();
                         }
-                        if (checkme.whatIsNaughtyLog.length() > 256) {
-                            checkme.whatIsNaughtyLog = String(checkme.whatIsNaughtyLog.c_str()).subString(0, 512).toCharArray();
+                        if (checkme.whatIsNaughtyLog.length() > 2048) {
+                            checkme.whatIsNaughtyLog = String(checkme.whatIsNaughtyLog.c_str()).subString(0, 2048).toCharArray();
                         }
                         peerconn.writeString("HTTP/1.0 302 Redirect\n");
                         peerconn.writeString("Location: ");
                         peerconn.writeString(o.access_denied_address.c_str());
                         peerconn.writeString("?DENIEDURL=");
-                        peerconn.writeString(url.toCharArray());
+                        peerconn.writeString(miniURLEncode(url.toCharArray()).c_str());
                         peerconn.writeString("&REASON=");
                         if (o.reporting_level == 1) {
                             peerconn.writeString(miniURLEncode(checkme.whatIsNaughty).c_str());
@@ -472,20 +569,23 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
                         peerconn.readyForOutput(10);
                         peerconn.writeString("HTTP/1.0 200 OK\n");
                         peerconn.writeString("Content-type: text/html\n\n");
-                        peerconn.writeString("<HTML><HEAD><TITLE>DansGuardian - Access Denied</TITLE></HEAD>");
-                        peerconn.writeString("<BODY><CENTER><H1>DansGuardian - Access Denied</H1></CENTER></BODY></HTML>");
+                        peerconn.writeString("<HTML><HEAD><TITLE>DansGuardian - ");
+                        peerconn.writeString(o.language_list.getTranslation(1)); // access denied
+                        peerconn.writeString("</TITLE></HEAD><BODY><CENTER><H1>DansGuardian - ");
+                        peerconn.writeString(o.language_list.getTranslation(1)); // access denied
+                        peerconn.writeString("</H1></CENTER></BODY></HTML>");
                     }
                     else if (o.reporting_level == -1) {  // stealth
                         checkme.isItNaughty = false;  // dont block
-                        #ifdef DGDEBUG
-                            std::cout << "STEALTHMODE!" << std::endl;
-                        #endif
                     }
                 } catch (exception& e) {}
                 if (checkme.isItNaughty) { // not stealth mode then
                     try {
                         peerconn.readyForOutput(10);  //as best a flush as I can
                     } catch (exception& e) {}
+                    try {
+                        proxysock.close();  // close connection to proxy
+                    } catch (exception& e) {}
                     return;  // we said no, so return
                 }
             }
@@ -509,7 +609,7 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
              std::cout << "sent header to client" << std::endl;
         #endif
         if (waschecked) {
-            decideHowToLog(clientuser, clientip, url.toCharArray(), exceptionreason, header.requesttype().toCharArray(), docsize, o.ll, false, isexception, o.log_exception_hits, docheader.iscontenttype("text"));
+            decideHowToLog(clientuser, clientip, url.toCharArray(), exceptionreason, header.requesttype().toCharArray(), docsize, o.ll, false, isexception, o.log_exception_hits, docheader.iscontenttype("text"), &thestart, cachehit, 200, mimetype);
 
             #ifdef DGDEBUG
                 std::cout << "sending body to client" << std::endl;
@@ -528,15 +628,19 @@ void ConnectionHandler::handleConnection(Socket peerconn) {
             fdt.tunnel(proxysock.getFD(), peerconn.getFD());
             docsize = fdt.throughput;
 
-            decideHowToLog(clientuser, clientip, url.toCharArray(), exceptionreason, header.requesttype().toCharArray(), docsize, o.ll, false, isexception, o.log_exception_hits, docheader.iscontenttype("text"));
+            decideHowToLog(clientuser, clientip, url.toCharArray(), exceptionreason, header.requesttype().toCharArray(), docsize, o.ll, false, isexception, o.log_exception_hits, docheader.iscontenttype("text"), &thestart, cachehit, 200, mimetype);
 
         }
     } catch (exception& e) {
         #ifdef DGDEBUG
             std::cout << "connection handler caught an exception" << std::endl;
         #endif
+        try {
+            proxysock.close();  // close connection to proxy
+        } catch (exception& e) {}
         return;
     }
+
     try {
         proxysock.close();  // close conection to squid
     } catch (exception& e) {}
@@ -568,9 +672,48 @@ std::string ConnectionHandler::miniURLEncode(std::string s) {
     return encoded;
 }
 
-void ConnectionHandler::doTheLogMan(std::string who, std::string from, std::string where, std::string what, std::string how, int size){
-    std::string logline,year,month,day,hour,min,sec,when,ssize;
-    String temp;
+void ConnectionHandler::doTheLogMan(std::string who, std::string from, std::string where, std::string what, std::string how, int size, struct timeval *thestart, bool cachehit, int code, std::string mimetype) {
+    std::string logline,year,month,day,hour,min,sec,when,ssize,duration, utime, hitmiss, hier;
+    struct timeval theend;
+    struct timezone notused;
+    gettimeofday(&theend, &notused);
+    long durationsecs, durationusecs;
+    durationsecs = theend.tv_sec - (*thestart).tv_sec;
+    durationusecs = theend.tv_usec - (*thestart).tv_usec;
+    durationusecs = (durationusecs / 1000) + durationsecs * 1000;
+    String temp = String((int)durationusecs);
+    while (temp.length() < 6) {
+        temp = " " + temp;
+    }
+    duration = temp.toCharArray();
+    temp = String((int)(theend.tv_usec / 1000));
+    while (temp.length() < 3) {
+        temp = "0" + temp;
+    }
+    if (temp.length() > 3) {
+        temp = "999";
+    }
+    utime = temp.toCharArray();
+    utime = "." + utime;
+    utime = String((int)theend.tv_sec).toCharArray() + utime;
+
+
+    if (code == 403) {
+        hitmiss = "TCP_DENIED/403";
+    }
+    else {
+        if (cachehit) {
+            hitmiss = "TCP_HIT/";
+            hitmiss += String(code).toCharArray();
+        }
+        else {
+           hitmiss = "TCP_MISS/";
+           hitmiss += String(code).toCharArray();
+        }
+    }
+    hier = "DEFAULT_PARENT/";
+    hier += o.proxy_ip;
+
     time_t tnow;  // to hold the result from time()
     struct tm *tmnow;  // to hold the result from localtime()
     time(&tnow);  // get the time after the lock so all entries in order
@@ -587,11 +730,19 @@ void ConnectionHandler::doTheLogMan(std::string who, std::string from, std::stri
     sec = temp.toCharArray();
     ssize = String(size).toCharArray();
     when = year + "." + month + "." + day + " " + hour + ":" + min + ":" + sec;
-    if (o.log_file_format == 1) {
-        logline = when +" "+ who + " " + from + " " + where + " " + what + " " + how + " " + ssize + "\n";
-    }
-    else {
-        logline = "\"" + when +"\",\""+ who + "\",\"" + from + "\",\"" + where + "\",\"" + what + "\",\"" + how + "\",\"" + ssize + "\"\n";
+    switch(o.log_file_format){
+        case 4:
+            logline = when +"\t"+ who + "\t" + from + "\t" + where + "\t" + what + "\t" + how + "\t" + ssize + "\n";
+            break;
+        case 3:
+            logline = utime + " " + duration + " " + from + " " + hitmiss + " " + ssize + " " + how + " " + where + " " + who + " " + hier + " " + mimetype + "\n";
+            break;
+
+        case 2:
+            logline = "\"" + when +"\",\""+ who + "\",\"" + from + "\",\"" + where + "\",\"" + what + "\",\"" + how + "\",\"" + ssize + "\"\n";
+            break;
+        default:
+            logline = when +" "+ who + " " + from + " " + where + " " + what + " " + how + " " + ssize + "\n";
     }
     UDSocket ipcsock;
     if (ipcsock.getFD() < 0) {
@@ -600,6 +751,7 @@ void ConnectionHandler::doTheLogMan(std::string who, std::string from, std::stri
     }
     if (ipcsock.connect(__IPC) < 0) {  // connect to dedicated logging proc
         syslog(LOG_ERR, "%s","Error connecting via ipc to log");
+        ipcsock.close();
         return;
     }
     ipcsock.writeString(logline.c_str());
@@ -608,7 +760,7 @@ void ConnectionHandler::doTheLogMan(std::string who, std::string from, std::stri
 
 
 
-void ConnectionHandler::decideHowToLog(std::string who, std::string from, std::string where, std::string what, std::string how, int size, int loglevel, bool isnaughty, bool isexception, int logexceptions, bool istext) {
+void ConnectionHandler::decideHowToLog(std::string who, std::string from, std::string where, std::string what, std::string how, int size, int loglevel, bool isnaughty, bool isexception, int logexceptions, bool istext, struct timeval *thestart, bool cachehit, int code, std::string mimetype) {
 
     if (loglevel == 0) {
         return;
@@ -630,6 +782,87 @@ void ConnectionHandler::decideHowToLog(std::string who, std::string from, std::s
          || isnaughty
          || loglevel == 3
          || (loglevel == 2 && istext)) {
-        doTheLogMan(who, from, where, what, how, size);
+        doTheLogMan(who, from, where, what, how, size, thestart, cachehit, code, mimetype);
     }
 }
+
+
+
+bool ConnectionHandler::wasClean(String url) {
+    url = url.after("://");
+    UDSocket ipcsock;
+    if (ipcsock.getFD() < 0) {
+        syslog(LOG_ERR, "%s","Error creating ipc socket to url cache");
+        return false;
+    }
+    if (ipcsock.connect(__URLIPC) < 0) {  // conn to dedicated url cach proc
+        syslog(LOG_ERR, "%s","Error connecting via ipc to url cache");
+        return false;
+    }
+    url += "\n";
+    char* reply = new char[8];
+    #ifdef DGDEBUG
+        std::cout << "sending clean request:" << url.toCharArray() << std::endl;
+    #endif
+    try {
+        ipcsock.writeString(url.toCharArray());  // throws on err
+    }
+    catch (exception& e) {
+        #ifdef DGDEBUG
+            std::cerr << "Exception writing to url cache" << std::endl;
+            std::cerr << e.what() << std::endl;
+        #endif
+        syslog(LOG_ERR, "%s","Exception writing to url cache");
+        syslog(LOG_ERR, "%s", e.what());
+    }
+    try {
+        ipcsock.getline(reply, 7, 6);  // throws on err
+    }
+    catch (exception& e) {
+        #ifdef DGDEBUG
+            std::cerr << "Exception reading from url cache" << std::endl;
+            std::cerr << e.what() << std::endl;
+        #endif
+        syslog(LOG_ERR, "%s","Exception reading from url cache");
+        syslog(LOG_ERR, "%s", e.what());
+    }
+    ipcsock.close();
+    if (reply[0] == 'Y') {
+        delete[] reply;
+        return true;
+    }
+    delete[] reply;
+    return false;
+}
+
+
+void ConnectionHandler::addToClean(String url) {
+    url = url.after("://");
+    UDSocket ipcsock;
+    if (ipcsock.getFD() < 0) {
+        syslog(LOG_ERR, "%s","Error creating ipc socket to url cache");
+        return;
+    }
+    if (ipcsock.connect(__URLIPC) < 0) {  // conn to dedicated url cach proc
+        syslog(LOG_ERR, "%s","Error connecting via ipc to url cache");
+        #ifdef DGDEBUG
+            std::cout << "Error connecting via ipc to url cache" << std::endl;
+        #endif
+        return;
+    }
+    url += "\n";
+    url = "A " + url;
+    try {
+        ipcsock.writeString(url.toCharArray());  // throws on err
+    }
+    catch (exception& e) {
+        #ifdef DGDEBUG
+            std::cerr << "Exception adding to url cache" << std::endl;
+            std::cerr << e.what() << std::endl;
+        #endif
+        syslog(LOG_ERR, "%s","Exception adding to url cache");
+        syslog(LOG_ERR, "%s", e.what());
+    }
+    ipcsock.close();
+}
+
