@@ -24,12 +24,15 @@
 #ifdef __BSD  // A bodge to get round the problem of a truncated UDS file
 	#include <sys/wait.h>
 	#define __IPC "/tmp/.dguardianipcc"
+	#define __URLIPC "/tmp/.dguardianurlipcc"
 #else
 	#include <wait.h>
 	#define __IPC "/tmp/.dguardianipc"
+	#define __URLIPC "/tmp/.dguardianurlipc"
 #endif
 #include <sys/stat.h>
 #include "ConnectionHandler.hpp"
+#include "DynamicURLList.hpp"
 #include "String.hpp"
 #include "Socket.hpp"
 #include "UDSocket.hpp"
@@ -41,10 +44,15 @@
 #include <pwd.h>
 #include <cerrno>
 #include <unistd.h>
-#include <istream.h>
 #include <fstream>
 #include <sys/time.h>
 
+#ifdef __GCCVER3
+    #include <istream>
+#else
+    #include <istream.h>
+#endif
+
 extern OptionContainer o;
 extern bool isDaemonised;
 
@@ -52,11 +60,14 @@ int FatController::controlIt(int pidfilefd) {
 
     pid_t childpid;  // to hold the child process pid
     pid_t loggerpid;  // to hold the logging process pid
+    pid_t urllistpid = 0;  // to hold the logging process pid (zero to stop
+                           // warning
     int rc, serversockfd;
 
     Socket serversock;  // the socket we will listen on for connections
     UDSocket ipcsock;  // the unix domain socket to be used for ipc with
                        // the forked children
+    UDSocket urllistsock;
 
     serversockfd = serversock.getFD();
 
@@ -99,6 +110,8 @@ int FatController::controlIt(int pidfilefd) {
     }
     string ipcfilename = "/tmp/.dguardianipc"; // Needs deleting if its there
     unlink(ipcfilename.c_str()); // this would normally be in a -r situation.
+    ipcfilename = "/tmp/.dguardianurlipc"; // Needs deleting if its there
+    unlink(ipcfilename.c_str()); // this would normally be in a -r situation.
 
     if (ipcsock.bind(__IPC)) {  // bind to file
         if (!isDaemonised) {
@@ -108,6 +121,17 @@ int FatController::controlIt(int pidfilefd) {
         return 1;
     }
 
+    if (o.url_cache_number > 0) {
+        if (urllistsock.bind(__URLIPC)) {  // bind to file
+            if (!isDaemonised) {
+                std::cerr << "Error binding urllistsock server file (try using the SysV to stop DansGuardian then try starting it again or doing an 'rm /tmp/.dguardianurlipc')." << std::endl;
+            }
+            syslog(LOG_ERR, "%s","Error binding urllistsock server file (try using the SysV to stop DansGuardian then try starting it again or doing an 'rm /tmp/.dguardianurlipc').");
+            return 1;
+        }
+    }
+
+
     if (serversock.listen(256)) {  // set it to listen mode with a kernel
                                   // queue of 256 backlog connections
         if (!isDaemonised) {
@@ -117,8 +141,8 @@ int FatController::controlIt(int pidfilefd) {
         return 1;
     }
 
-    if (ipcsock.listen(128)) {  // set it to listen mode with a kernel
-                                  // queue of 128 backlog connections
+    if (ipcsock.listen(256)) {  // set it to listen mode with a kernel
+                                  // queue of 256 backlog connections
         if (!isDaemonised) {
             std::cerr << "Error listening to ipc server file" << std::endl;
         }
@@ -126,6 +150,17 @@ int FatController::controlIt(int pidfilefd) {
         return 1;
     }
 
+    if (o.url_cache_number > 0) {
+        if (urllistsock.listen(256)) {  // set it to listen mode with a kernel
+                                  // queue of 256 backlog connections
+            if (!isDaemonised) {
+                std::cerr << "Error listening to url ipc server file" << std::endl;
+            }
+            syslog(LOG_ERR, "%s","Error listening to url ipc server file");
+            return 1;
+        }
+    }
+
     Socket peersock;  // the socket which will contain the connection
 
     if (peersock.getFD() < 0) {
@@ -177,6 +212,10 @@ int FatController::controlIt(int pidfilefd) {
 
     if (loggerpid == 0) {  // ma ma!  i am the child
         serversock.close();  // we don't need our copy of this so close it
+        urllistsock.close();  // we don't need our copy of this so close it
+        if (o.url_cache_number > 0) {
+            urllistsock.close();  // we don't need our copy of this so close it
+        }
         logListener(ipcsock, o.loglocation, o.logconerror);
         #ifdef DGDEBUG
             std::cout << "Log listener exiting" << std::endl;
@@ -184,8 +223,27 @@ int FatController::controlIt(int pidfilefd) {
         _exit(0); // is reccomended for child and daemons to use this instead
     }
 
+
+    if (o.url_cache_number > 0) {
+        urllistpid = fork(); // make a child processes copy of self to be logger
+
+        if (urllistpid == 0) {  // ma ma!  i am the child
+            serversock.close();  // we don't need our copy of this so close it
+            ipcsock.close();  // we don't need our copy of this so close it
+
+            urlListListener(urllistsock, o.logconerror);
+            #ifdef DGDEBUG
+                std::cout << "URL List listener exiting" << std::endl;
+            #endif
+            _exit(0); // is reccomended for child and daemons to use this instead
+        }
+    }
+
     // I am the parent process here onwards.
 
+    if (o.url_cache_number > 0) {
+        urllistsock.close();  // we don't need our copy of this so close it
+    }
     ipcsock.close();  // we don't need our copy of this so close it
 
     memset(&sa, 0, sizeof(sa));
@@ -419,6 +477,9 @@ int FatController::controlIt(int pidfilefd) {
     if (reloadconfig) {
         reloadconfig = false;
         ::kill(loggerpid, SIGTERM);  // get rid of logger
+        if (o.url_cache_number > 0) {
+            ::kill(urllistpid, SIGTERM);  // get rid of url cache
+        }
         return 2;
     }
     if (ttg) {
@@ -452,7 +513,7 @@ bool FatController::testProxy(std::string proxyip, int proxyport, bool report) {
     if (conn) {
         if (report) {
             if (!isDaemonised) {
-                std::cerr << "Error creating connecting to test proxy" << std::endl;
+                std::cerr << "Error creating connection to test proxy" << std::endl;
             }
             syslog(LOG_ERR, "%s","Error creating connection to test proxy");
         }
@@ -502,6 +563,7 @@ bool FatController::daemonise(int pidfilefd) {
 
 int FatController::logListener(UDSocket ipcsock, std::string loglocation, int logconerror) {
     UDSocket ipcpeersock;  // the socket which will contain the ipc connection
+    ipcpeersock.close(); // we won't need its fd as we'll be accept()ing
     int rc, ipcsockfd;
     char* logline = new char[4096];
 
@@ -547,6 +609,7 @@ int FatController::logListener(UDSocket ipcsock, std::string loglocation, int lo
             #endif
             ipcpeersock = ipcsock.accept();
             if (ipcpeersock.getFD() < 0) {
+                ipcpeersock.close();
                 if (logconerror == 1) {
                     syslog(LOG_ERR, "%s","Error accepting ipc. (Ignorable)");
                 }
@@ -556,6 +619,7 @@ int FatController::logListener(UDSocket ipcsock, std::string loglocation, int lo
             try {
                 rc = ipcpeersock.getline(logline, 4096, 3);  // throws on err
             } catch (exception& e) {
+                ipcpeersock.close();
                 if (logconerror == 1) {
                     syslog(LOG_ERR, "%s","Error reading ipc. (Ignorable)");
                 }
@@ -609,3 +673,111 @@ extern "C" {  // The kernel knows nothing of objects so
     }
 }
 
+int FatController::urlListListener(UDSocket ipcsock, int logconerror) {
+    #ifdef DGDEBUG
+            std::cout << "url listener started" << std::endl;
+    #endif
+    UDSocket ipcpeersock;  // the socket which will contain the ipc connection
+    ipcpeersock.close(); // we won't need its fd as we'll be accept()ing
+    int rc, ipcsockfd;
+    char* logline = new char[4096];
+    String reply;
+    DynamicURLList urllist;
+    #ifdef DGDEBUG
+            std::cout << "setting url list size-age:" << o.url_cache_number << "-" << o.url_cache_age << std::endl;
+    #endif
+    urllist.setListSize(o.url_cache_number, o.url_cache_age);
+    ipcsockfd = ipcsock.getFD();
+    #ifdef DGDEBUG
+            std::cout << "url ipcsockfd:" << ipcsockfd << std::endl;
+    #endif
+
+    fd_set fdSet;  // our set of fds (only 1) that select monitors for us
+    fd_set fdcpy;  // select modifes the set so we need to use a copy
+    FD_ZERO(&fdSet);  // clear the set
+    FD_SET(ipcsockfd, &fdSet);  // add ipcsock to the set
+
+    #ifdef DGDEBUG
+            std::cout << "url listener entering select()" << std::endl;
+    #endif
+    while (true) {  // loop, essentially, for ever
+
+        fdcpy = fdSet;  // take a copy
+
+        rc = select(ipcsockfd + 1, &fdcpy, NULL, NULL, NULL);  // block
+                                                  // until something happens
+        #ifdef DGDEBUG
+            std::cout << "url listener select returned" << std::endl;
+        #endif
+        if (rc < 0) {  // was an error
+            if (errno == EINTR) {
+                continue;  // was interupted by a signal so restart
+            }
+            if (logconerror == 1) {
+                syslog(LOG_ERR, "%s","url ipc rc<0. (Ignorable)");
+            }
+            continue;
+        }
+        if (FD_ISSET(ipcsockfd, &fdcpy)) {
+            #ifdef DGDEBUG
+                std::cout << "received an url request" << std::endl;
+            #endif
+            ipcpeersock = ipcsock.accept();
+            if (ipcpeersock.getFD() < 0) {
+                ipcpeersock.close();
+                if (logconerror == 1) {
+                    #ifdef DGDEBUG
+                        std::cout << "Error accepting url ipc. (Ignorable)" << std::endl;
+                    #endif
+                    syslog(LOG_ERR, "%s","Error accepting url ipc. (Ignorable)");
+                }
+                continue; // if the fd of the new socket < 0 there was error
+                          // but we ignore it as its not a problem
+            }
+            try {
+                rc = ipcpeersock.getline(logline, 4096, 3);  // throws on err
+            } catch (exception& e) {
+                ipcpeersock.close();
+                if (logconerror == 1) {
+                    #ifdef DGDEBUG
+                        std::cout << "Error reading ip ipc. (Ignorable)" << std::endl;
+                    #endif
+                    syslog(LOG_ERR, "%s","Error reading ip ipc. (Ignorable)");
+                }
+                continue;
+            }
+            if (logline[0] == 'A' && logline[1] == ' ') {
+                ipcpeersock.close();  // close the connection
+                urllist.addEntry(logline + 2);
+                #ifdef DGDEBUG
+                    std::cout << "url add request:" << logline << std::endl;
+                #endif
+                continue;
+            }
+            if (urllist.inURLList(logline)) {
+                reply = "Y\n";
+            }
+            else {
+                reply = "N\n";
+            }
+            try {
+                ipcpeersock.writeString(reply.toCharArray());
+            } catch (exception& e) {
+                if (logconerror == 1) {
+                    syslog(LOG_ERR, "%s","Error writing url ipc. (Ignorable)");
+                }
+                continue;
+            }
+            ipcpeersock.close();  // close the connection
+            #ifdef DGDEBUG
+                std::cout << "url list reply:" << reply << std::endl;
+            #endif
+            continue;  // go back to listening
+        }
+   }
+    delete[] logline;
+    ipcsock.close();  // be nice and neat
+    return 1; // It is only possible to reach here with an error
+
+}
+
