@@ -138,6 +138,9 @@ String HTTPHeader::getcontenttype() {  // most self explanetory
         }
     }
     mimetype.toLower();
+    if (mimetype.length() < 1) {
+        mimetype = "-";
+    }
     return mimetype;
 }
 
@@ -161,10 +164,8 @@ std::string HTTPHeader::getXForwardedForIP() {
 }
 
 
+
 bool HTTPHeader::malformedURL(String url) {
-    #ifdef DGDEBUG
-        std::cout << "checking url:" << url << std::endl;
-    #endif
     String host = url.after("://");
     if (host.contains("/")) {
         host = host.before("/");
@@ -172,6 +173,9 @@ bool HTTPHeader::malformedURL(String url) {
     if (host.length() < 2) {
         return true;
     }
+    if (host.contains("..")) {
+        return true;
+    }
     int i, len;
     unsigned char c;
     len = host.length();
@@ -191,8 +195,8 @@ bool HTTPHeader::malformedURL(String url) {
 //  GET http://foo.bar:80/ HTML/1.0 (if :80 is omitted 80 is assumed)
 // or:
 //  GET / HTML/1.0
-//  Host: foo.bar
-//  Port: 80
+//  Host: foo.bar (optional header in HTTP/1.0, but like HTTP/1.1, we require it!)
+//  Port: 80 (not a standard header; do any clients send it?)
 // or:
 //  CONNECT foo.bar:443  HTTP/1.1
 // So we need to handle all 3
@@ -200,7 +204,7 @@ bool HTTPHeader::malformedURL(String url) {
 String HTTPHeader::url() {
     String hostname;
     String answer = header[0].after(" ");
-    if (answer.after(" ").contains("HTTP/")) {
+    if (answer.after(" ").startsWith("HTTP/")) {
         answer = answer.before(" HTTP/");
     }
     else {
@@ -211,9 +215,7 @@ String HTTPHeader::url() {
             answer = "https://" + answer;
         }
     }
-    if (!answer.after("://").contains("/")) {
-            answer += "/";  // needed later on so correct host is extracted
-    }
+
     if (answer.length()) {
         int i;
         if (answer[0] == '/') {  // must be the latter above
@@ -232,10 +234,9 @@ String HTTPHeader::url() {
             }
         }
         else {  // must be in the form GET http://foo.bar:80/ HTML/1.0
-            #ifdef DGDEBUG
-                std::cout << "url must be in the form GET http://foo.bar:80/ HTML/1.0:" << std::endl;
-                std::cout << answer << std::endl;
-            #endif
+            if (!answer.after("://").contains("/")) {
+                answer += "/";  // needed later on so correct host is extracted
+            }
             String protocol = answer.before("://");
             hostname = answer.after("://");
             String url = hostname.after("/");
@@ -267,11 +268,14 @@ String HTTPHeader::decode(String s) {
         std::cout << "decoding url" << std::endl;
     #endif
     RegExp re;
-    re.match("%[0-9a-fA-F][0-9a-fA-F]", s.toCharArray());
+
     #ifdef DGDEBUG
         std::cout << "matches:" << re.numberOfMatches() << std::endl;
     #endif
     if (!re.matched()) {return s;}  // exit if not found
+      #ifdef DGDEBUG
+        std::cout << "removing %XX" << std::endl;
+    #endif
     int match;
     int offset;
     int pos = 0;
@@ -337,9 +341,9 @@ String HTTPHeader::hexToChar(String n) {
     return n;
 }
 
+
 String HTTPHeader::requesttype() {  // GET,HEAD,PUT,etc - what?
     return header[0].before(" ");
-
 }
 
 
@@ -567,6 +571,16 @@ void HTTPHeader::removeEncoding(int newlen) {
     }
 }
 
+void HTTPHeader::setContentLength(int newlen) {
+    for (int i = 0; i < (signed)header.size(); i++) {
+        if (header[i].startsWith("Content-Length:")) {
+            header[i] = "Content-Length: " + String(newlen);
+        }
+    }
+}
+
+
+
 bool HTTPHeader::isPostUpload() {
     bool answer = false;
     int postlen = postdata.buffer_length;
