@@ -97,12 +97,8 @@ bool ListContainer::readPhraseList(const char* filename, bool isexception) {
         return false;
     }
     if (len < 4) {
-        if (!isDaemonised) {
-            std::cerr << "File too small (less than 4 bytes - is it corrupt?): " << filename << std::endl;
-        }
-        syslog(LOG_ERR, "%s","File too small (less than 4 bytes - is it corrupt?): ");
-        syslog(LOG_ERR, "%s",filename);
-        return false;
+        return true; // its blank - perhaps due to webmin editing
+                    // just return
     }
     increaseMemoryBy(len);  // Allocate some memory to hold file
     ifstream listfile(filename, ios::in);  // open the file for reading
@@ -215,60 +211,13 @@ void ListContainer::readPhraseListHelper2(String phrase, int type, int weighting
 
     // must be a combi if got here
 
-    if (addToItemListPhrase(phrase.toCharArray(), phrase.length(), type, weighting, true)) {
+    addToItemListPhrase(phrase.toCharArray(), phrase.length(), type, weighting, true);
 
-        return;
-    }
-
-
-    // actually this code should never run
-
-    // as it could only run if, well... you work it out
-
-    if (!isDaemonised) {
-        std::cerr << "This is STILL not happening!!!!!" << std::endl;
-    }
-
-    int i;
-    int index = 0;
-    std::string ss = phrase.toCharArray();
-    for (i = 0; i < items; i++) {
-        if (ss == getItemAt(i)) {
-            index = i;
-            break;
-        }
-    }
-    combilist.push_back(index);
 }
 
 
 bool ListContainer::addToItemListPhrase(char* s, int len, int type, int weighting, bool combi) {
-    int i, it;
-    std::string ss = s;
-    bool found = false;
-    for (i = 0; i < items; i++) {
-        if (ss == getItemAt(i)) {
-            found = true;
-            break;
-        }
-    }
-    if (found) {
-        it = itemtype[i];
-        if (combi) {
-                combilist.push_back(i);
-                return true;
-        }
-        if (it > 9) {  // exists as a combi entry already
-            // if got here existing entry must be a combi AND
-            // new entry is not a combi so we overwrite the
-            // existing values as combi values and types are
-            // stored in the combilist
-            itemtype[i] = type;
-            weight[i] = weighting;
-            return true;
-        }
-        return false;
-    }
+    int i;
     list.push_back(data_length);
     for(i = 0; i < len; i++) {
         data[data_length + i] = s[i];
@@ -288,6 +237,7 @@ bool ListContainer::addToItemListPhrase(char* s, int len, int type, int weightin
 bool ListContainer::readItemList(const char* filename, bool startswith, int filters) {
     std::string linebuffer;
     RegExp re;
+    re.comp("^.*\\:[0-9]+\\/.*");
     #ifdef DGDEBUG
         std::cout << filename << std::endl;
     #endif
@@ -313,13 +263,9 @@ bool ListContainer::readItemList(const char* filename, bool startswith, int filt
         syslog(LOG_ERR, "%s",filename);
         return false;
     }
-    if (len < 5) {
-        if (!isDaemonised) {
-            std::cerr << "File too small (less than 5 bytes - is it corrupt?): " << filename << std::endl;
-        }
-        syslog(LOG_ERR, "%s","File too small (less than 5 bytes - is it corrupt?):");
-        syslog(LOG_ERR, "%s",filename);
-        return false;
+    if (len < 4) {
+        return true;  // its blank - perhaps due to webmin editing
+                    // just return
     }
     increaseMemoryBy(len);  // Allocate some memory to hold file
     ifstream listfile(filename, ios::in);
@@ -335,6 +281,10 @@ bool ListContainer::readItemList(const char* filename, bool startswith, int filt
     while (!listfile.eof()) {
         getline(listfile, linebuffer);
         if (linebuffer.length() < 2) continue;  // its jibberish
+
+//       cout << "linebuffer.length():" << linebuffer.length() << endl;
+
+
         if (linebuffer[0] == '#') continue;  // its a comment
         temp = (char*)linebuffer.c_str();
         if (temp.contains("#")) {
@@ -358,7 +308,7 @@ bool ListContainer::readItemList(const char* filename, bool startswith, int filt
         }
         if (filters == 1) {  // remove port addresses
             if (temp.contains(":")) {  // quicker than full regexp
-                if (re.match("^.*\\:[0-9]+\\/.*", temp.toCharArray())) {
+                if (re.match(temp.toCharArray())) {
                     hostname = temp.before(":");
                     url = temp.after("/");
                     temp = hostname + "/" + url;
@@ -366,6 +316,7 @@ bool ListContainer::readItemList(const char* filename, bool startswith, int filt
             }
         }
         temp.toLower();  // tidy up
+//cout << temp << " " << temp.length() << endl;
         addToItemList(temp.toCharArray(), temp.length());  // add to unsorted
                                                            // list
     }
@@ -498,6 +449,7 @@ bool ListContainer::createCacheFile(const char* filename) {
 
 
 void ListContainer::makeGraph() {
+    if (data_length == 0) return;
     graphdata = new int[64 * data_length];
     graphitems++;
     memset(graphdata, 0, sizeof(int) * 64 * data_length);
@@ -735,7 +687,7 @@ void ListContainer::graphAdd(String s, int inx, int item) {
     unsigned char c;
     bool found = false;
     String t;
-    int i;
+    int i, px, it;
     int numlinks;
     for (i = 0; i < graphdata[inx * 64 + 2]; i++) {
         c = (unsigned char)graphdata[(graphdata[inx * 64 +  4 + i]) * 64];
@@ -746,6 +698,23 @@ void ListContainer::graphAdd(String s, int inx, int item) {
                 graphAdd(t, graphdata[inx * 64 + 4 + i], item);
             }
             found = true;
+            if (t.length() == 0) { // this means the phrase is already there
+                                   // as part of an existing phrase
+                px = graphdata[(graphdata[inx * 64 +  4 + i]) * 64 + 1];
+                if (px == 1) {  // the exact phrase is already there
+                    px = graphdata[(graphdata[inx * 64 +  4 + i]) * 64 + 3];
+                    it = itemtype[px];
+                    if (it > 9 && itemtype[item] < 10) {
+                        // exists as a combi entry already
+                        // if got here existing entry must be a combi AND
+                        // new entry is not a combi so we overwrite the
+                        // existing values as combi values and types are
+                        // stored in the combilist
+                        itemtype[px] = itemtype[item];
+                        weight[px] = weight[item];
+                    }
+                }
+            }
         }
     }
     if (!found) {
@@ -865,7 +834,7 @@ bool ListContainer::readProcessedItemList(const char* filename) {
         syslog(LOG_ERR, "%s",filename);
         return false;
     }
-    increaseMemoryBy(len);
+    increaseMemoryBy(len + 2);
     ifstream listfile(filename, ios::in);
     if (!listfile.good()) {
         if (!isDaemonised) {
@@ -1087,9 +1056,6 @@ bool ListContainer::isCacheFileNewer(const char* filename) {
     linebuffer = filename;
     linebuffer += ".processed";
     int cachedate = getFileDate(linebuffer.c_str());
-    if (cachedate < bannedlistdate) {
-        return false;  // cache file is older than list file
-    }
     while (!listfile.eof()) {
         getline(listfile, linebuffer);
         if (linebuffer.length() < 3) continue;
@@ -1105,6 +1071,9 @@ bool ListContainer::isCacheFileNewer(const char* filename) {
         }
     }
     listfile.close();
+    if (cachedate < bannedlistdate) {
+        return false;  // cache file is older than list file
+    }
     return true;
 }
 
