@@ -147,7 +147,9 @@ void NaughtyFilter::checkphrase(char* file, int l) {
                 if (type == -1) {  // combination exception
                     isItNaughty = false;
                     isException = true;
-                    whatIsNaughtyLog = "Combination exception phrase found: " + combisofar;
+                    whatIsNaughtyLog = o.language_list.getTranslation(605);
+                    // Combination exception phrase found:
+                    whatIsNaughtyLog += combisofar;
                     whatIsNaughty = "";
                     return;
                 }
@@ -265,20 +267,26 @@ void NaughtyFilter::checkphrase(char* file, int l) {
     #endif
     if (isException) {
         isItNaughty = false;
-        whatIsNaughtyLog = "Exception phrase found: " + exceptionphrase;
+        whatIsNaughtyLog = o.language_list.getTranslation(604);
+        // Exception phrase found:
+        whatIsNaughtyLog += exceptionphrase;
         whatIsNaughty = "";
         return;
     }
 
     if (isItNaughty) {
-        whatIsNaughtyLog = "Banned Phrase found: " + bannedphrase;
-        whatIsNaughty = "Banned phrase found.";
+        whatIsNaughtyLog = o.language_list.getTranslation(300);
+        // Banned Phrase found:
+        whatIsNaughtyLog += bannedphrase;
+        whatIsNaughty = o.language_list.getTranslation(301);
+        // Banned phrase found.
         return;
     }
 
     if (weighting > o.naughtyness_limit) {
         isItNaughty = true;
-        whatIsNaughtyLog = "Weighted phrase limit of ";
+        whatIsNaughtyLog = o.language_list.getTranslation(402);
+        // Weighted phrase limit of
         whatIsNaughtyLog += String(o.naughtyness_limit).toCharArray();
         whatIsNaughtyLog += " : ";
         whatIsNaughtyLog += String(weighting).toCharArray();
@@ -287,13 +295,17 @@ void NaughtyFilter::checkphrase(char* file, int l) {
             whatIsNaughtyLog += weightedphrase;
             whatIsNaughtyLog += ")";
         }
-        whatIsNaughty = "Weighted phrase limit exceeded.";
+        whatIsNaughty = o.language_list.getTranslation(403);
+        // Weighted phrase limit exceeded.
         return;
     }
     if (bannedcombifound) {
         isItNaughty = true;
-        whatIsNaughtyLog = "Banned combination phrase found: " + combifound;
-        whatIsNaughty = "Banned combination phrase found.";
+        whatIsNaughtyLog = o.language_list.getTranslation(400);
+        // Banned combination phrase found:
+        whatIsNaughtyLog += combifound;
+        whatIsNaughty = o.language_list.getTranslation(401);
+        // Banned combination phrase found.
         return;
     }
 
@@ -307,39 +319,54 @@ void NaughtyFilter::checkphrase(char* file, int l) {
 // lower case.
 void NaughtyFilter::checkPICS(char* file, int l) {
     file[l] = 0;  // to ensure end of c-string marker is there
-    RegExp re;
-    re.match("pics-label\"[ \t]*content=[\'\"]([^>]*)[\'\"]", file);
-    if (!re.matched()) {return;}  // exit if not found
-    for (int i = 0; i < re.numberOfMatches(); i++) {
-        checkPICSrating(re.result(i));  // pass on result for further
+//    RegExp re;
+//    re.comp("pics-label\"[ \t]*content=[\'\"]([^>]*)[\'\"]");
+    o.pics1.match(file);
+    if (!o.pics1.matched()) {return;}  // exit if not found
+    for (int i = 0; i < o.pics1.numberOfMatches(); i++) {
+        checkPICSrating(o.pics1.result(i));  // pass on result for further
                                               // tests
     }
 }
 
 void NaughtyFilter::checkPICSrating(std::string label) {
-    RegExp re;
-    re.match("[r|{ratings}] *\\(([^\\)]*)\\)", label.c_str());
-    if (!re.matched()) {return;}  // exit if not found
+//    RegExp re;
+//    re.comp("[r|{ratings}] *\\(([^\\)]*)\\)");
+    o.pics2.match(label.c_str());
+    if (!o.pics2.matched()) {return;}  // exit if not found
     String lab = label.c_str();  // convert to a String for easy manip
     String r;
-    for (int i = 0; i < re.numberOfMatches(); i++) {
-        r = re.result(i).c_str();  // ditto
+    String service;
+    for (int i = 0; i < o.pics2.numberOfMatches(); i++) {
+        r = o.pics2.result(i).c_str();  // ditto
         r = r.after("(");
         r = r.before(")");  // remove the brackets
 
-        if (lab.contains("safesurf")) {checkPICSratingSafeSurf(r);}
+        // Only check the substring of lab that is between
+	// the start of lab (or the end of the previous match)
+	// and the start of this rating.
+	// It is possible to have multiple ratings in one pics-label.
+	// This is done on e.g. http://www.jesusfilm.org/
+
+        if (i == 0) {
+	    service = lab.subString(0, o.pics2.offset(i));
+	} else {
+	    service = lab.subString(o.pics2.offset(i-1)+o.pics2.length(i-1), o.pics2.offset(i));
+	}
+
+        if (service.contains("safesurf")) {checkPICSratingSafeSurf(r);}
         if (isItNaughty) {return;}
-        if (lab.contains("evaluweb")) {checkPICSratingevaluWEB(r);}
+        if (service.contains("evaluweb")) {checkPICSratingevaluWEB(r);}
         if (isItNaughty) {return;}
-        if (lab.contains("microsys")) {checkPICSratingCyberNOT(r);}
+        if (service.contains("microsys")) {checkPICSratingCyberNOT(r);}
         if (isItNaughty) {return;}
-        if (lab.contains("icra")) {checkPICSratingICRA(r);}
+        if (service.contains("icra")) {checkPICSratingICRA(r);}
         if (isItNaughty) {return;}
-        if (lab.contains("rsac")) {checkPICSratingRSAC(r);}
+        if (service.contains("rsac")) {checkPICSratingRSAC(r);}
         if (isItNaughty) {return;}
-        if (lab.contains("weburbia")) {checkPICSratingWeburbia(r);}
+        if (service.contains("weburbia")) {checkPICSratingWeburbia(r);}
         if (isItNaughty) {return;}
-        if (lab.contains("vancouver")) {checkPICSratingVancouver(r);}
+        if (service.contains("vancouver")) {checkPICSratingVancouver(r);}
 
           // check label for word denoting rating system then pass on to the
           // appropriate function the rating String.
@@ -499,7 +526,10 @@ void NaughtyFilter::checkPICSagainstoption(String s, char* l, int opt, std::stri
             i = s.toInteger();  // convert the value in a String to an integer
             if (opt < i) {  // check its value against the option in config file
                 isItNaughty = true;  // must be over limit
-                whatIsNaughty = m + " PICS labeling level exceeded on the above site.";  // report
+                whatIsNaughty = m + " ";
+                whatIsNaughty += o.language_list.getTranslation(1000);
+                // PICS labeling level exceeded on the above site.
+
                 whatIsNaughtyLog = whatIsNaughty;
             }
         }
@@ -507,3 +537,4 @@ void NaughtyFilter::checkPICSagainstoption(String s, char* l, int opt, std::stri
 }
 
 
+
