@@ -47,8 +47,13 @@ void OptionContainer::reset() {
     banned_regexpurl_list.reset();
     weighted_phrase_list.reset();
     html_template.reset();
+    content_regexp_list.reset();
+    language_list.reset();
     banned_phrase_list_index.clear();
     conffile.clear();
+    banned_regexpurl_list_comp.clear();
+    content_regexp_list_comp.clear();
+    content_regexp_list_rep.clear();
 }
 
 bool OptionContainer::read(std::string filename) {
@@ -116,8 +121,20 @@ bool OptionContainer::read(std::string filename) {
         if (!realitycheck(String(max_children), 1, 3, "maxchildren"))
             { return false; }  // check its a reasonable value
         max_upload_size = findoptionI("maxuploadsize") * 1024;
-        if (!realitycheck(String(max_upload_size), 1, 6, "maxuploadsize"))
+        if (!realitycheck(String(max_upload_size), 1, 8, "maxuploadsize"))
             { return false; }  // check its a reasonable value
+        max_content_filter_size = findoptionI("maxcontentfiltersize") * 1024 * 1024;
+        if (!realitycheck(String(max_content_filter_size), 1, 8, "maxcontentfiltersize"))
+            { return false; }  // check its a reasonable value
+
+        url_cache_number = findoptionI("urlcachenumber");
+        if (!realitycheck(String(url_cache_number), 1, 5, "urlcachenumber"))
+            { return false; }  // check its a reasonable value
+
+        url_cache_age = findoptionI("urlcacheage");
+        if (!realitycheck(String(url_cache_age), 1, 5, "urlcacheage"))
+            { return false; }  // check its a reasonable value
+
         filter_port = findoptionI("filterport");
         if (!realitycheck(String(filter_port), 2, 6, "filterport"))
             { return false; }  // check its a reasonable value
@@ -136,7 +153,7 @@ bool OptionContainer::read(std::string filename) {
         log_file_format = findoptionI("logfileformat");
         if (!realitycheck(log_file_format, 1, 1, "logfileformat"))
             { return false; }  // etc
-        if (log_file_format < 1 || log_file_format > 2) {
+        if (log_file_format < 1 || log_file_format > 4) {
             log_file_format = 1;
         }
         if (findoptionS("enablePICS") == "off") {
@@ -162,7 +179,7 @@ bool OptionContainer::read(std::string filename) {
             { return false; }
         html_template_location = findoptionS("htmltemplate");
 
-        if (findoptionS("forwarded_for") == "on") {
+        if (findoptionS("forwardedfor") == "on") {
            forwarded_for = 1;
         }
         else {
@@ -210,6 +227,7 @@ bool OptionContainer::read(std::string filename) {
         else {
            reverse_lookups = 0;
         }
+
         if (findoptionS("usexforwardedfor") == "on") {
 	   use_xforwardedfor = 1;
 	}
@@ -227,13 +245,15 @@ bool OptionContainer::read(std::string filename) {
         banned_site_list_location = findoptionS("bannedsitelist");
         banned_url_list_location = findoptionS("bannedurllist");
         banned_regexpurl_list_location = findoptionS("bannedregexpurllist");
+        content_regexp_list_location = findoptionS("contentregexplist");
         exceptions_site_list_location = findoptionS("exceptionsitelist");
         exceptions_url_list_location = findoptionS("exceptionurllist");
         exceptions_user_list_location = findoptionS("exceptionuserlist");
         exceptions_ip_list_location = findoptionS("exceptioniplist");
+        language_list_location = findoptionS("languagefile");
         access_denied_address = findoptionS("accessdeniedaddress");
         ada = access_denied_address.c_str();
-        ada = ada.after("http://");
+        ada = ada.after("://");
         ada.removeWhiteSpace();
         ada = ada.before("/"); // ada now contains the FQ host name of the
                                // server that serves the accessdenied.html
@@ -284,7 +304,7 @@ bool OptionContainer::read(std::string filename) {
         pics_icra_nuditykissing = findoptionI("ICRAnuditykissing");
         pics_icra_nudityartistic = findoptionI("ICRAnudityartistic");
         pics_icra_nudityeducational = findoptionI("ICRAnudityeducational");
-        pics_icra_nuditymedical = findoptionI("ICRAnuditymedica");
+        pics_icra_nuditymedical = findoptionI("ICRAnuditymedical");
         pics_icra_drugstobacco = findoptionI("ICRAdrugstobacco");
         pics_icra_drugsalcohol = findoptionI("ICRAdrugsalcohol");
         pics_icra_drugsuse = findoptionI("ICRAdrugsuse");
@@ -357,9 +377,24 @@ bool OptionContainer::read(std::string filename) {
         if (!readbulfile(banned_url_list_location.c_str())) {
             return false;
         }  // banned urls
+
+
         if (!readbreulfile(banned_regexpurl_list_location.c_str())) {
             return false;
         }  // banned reg exp urls
+
+        if (!readcrelfile(content_regexp_list_location.c_str())) {
+            return false;
+        }  // content replacement regular expressions
+
+        if (!precompileregexps()) {
+            return false;
+        }  // precompiled reg exps for speed
+
+        if (!language_list.readLanguageList(language_list_location.c_str())) {
+            return false;
+        }  // messages language file
+
         if (banned_site_list.inList("**")) {
             blanketblock = 1;
         }
@@ -395,15 +430,7 @@ bool OptionContainer::read(std::string filename) {
 
 bool OptionContainer::readbplfile(const char* banned, const char* exception, const char* weighted) {
 
-    bool result = banned_phrase_list.readPhraseList(banned, false);
-    if (!result) {
-        if (!isDaemonised) {
-            std::cerr << "Error opening bannedphraselist" << std::endl;
-        }
-        syslog(LOG_ERR, "%s","Error opening bannedphraselist");
-        return false;
-    }
-    result = banned_phrase_list.readPhraseList(exception, true);
+    bool result = banned_phrase_list.readPhraseList(exception, true);
     if (!result) {
         if (!isDaemonised) {
             std::cerr << "Error opening exceptionphraselist" << std::endl;
@@ -411,6 +438,14 @@ bool OptionContainer::readbplfile(const char* banned, const char* exception, con
         syslog(LOG_ERR, "%s","Error opening exceptionphraselist");
         return false;
     }
+    result = banned_phrase_list.readPhraseList(banned, false);
+    if (!result) {
+        if (!isDaemonised) {
+            std::cerr << "Error opening bannedphraselist" << std::endl;
+        }
+        syslog(LOG_ERR, "%s","Error opening bannedphraselist");
+        return false;
+    }
     if (weighted_phrase_mode > 0) {  // if zero wpl is deactivated
         result = banned_phrase_list.readPhraseList(weighted, false);
         if (!result) {
@@ -592,10 +627,60 @@ bool OptionContainer::readbreulfile(const char* filename) {
         syslog(LOG_ERR, "%s","Error opening bannedregexpurllist");
         return false;
     }
-    banned_regexpurl_list.startsWithSort();
+    RegExp r;
+    bool rv = true;
+    int len = banned_regexpurl_list.getListLength();
+
+    for(int i = 0; i < len; i++) {
+        rv = r.comp(banned_regexpurl_list.getItemAt(i).c_str());
+        if (rv == false) {
+            if (!isDaemonised) {
+                std::cerr << "Error compiling regexp:" << banned_regexpurl_list.getItemAt(i) << std::endl;
+            }
+            syslog(LOG_ERR, "%s", "Error compiling regexp:");
+            syslog(LOG_ERR, "%s", banned_regexpurl_list.getItemAt(i).c_str());
+        return false;
+        }
+        banned_regexpurl_list_comp.push_back(r);
+    }
     return true;
 }
 
+bool OptionContainer::readcrelfile(const char* filename) {
+    bool result = content_regexp_list.readItemList(filename, true, 0);
+    if (!result) {
+        if (!isDaemonised) {
+            std::cerr << "Error opening contentregexplist" << std::endl;
+        }
+        syslog(LOG_ERR, "%s","Error opening contentregexplist");
+        return false;
+    }
+    content_regexp_list.startsWithSort();
+    RegExp r;
+    bool rv = true;
+    String regexp;
+    String replacement;
+    for(int i = 0; i < content_regexp_list.getListLength(); i++) {
+        regexp = content_regexp_list.getItemAt(i).c_str();
+        replacement = regexp.after("\"->\"").before("\"");
+        regexp = regexp.after("\"").before("\"->\"");
+        if (replacement.length() < 1 || regexp.length() < 1) {
+            continue;
+        }
+        rv = r.comp(regexp.toCharArray());
+        if (rv == false) {
+            if (!isDaemonised) {
+                std::cerr << "Error compiling regexp:" << content_regexp_list.getItemAt(i) << std::endl;
+            }
+            syslog(LOG_ERR, "%s", "Error compiling regexp:");
+            syslog(LOG_ERR, "%s", content_regexp_list.getItemAt(i).c_str());
+        return false;
+        }
+        content_regexp_list_comp.push_back(r);
+        content_regexp_list_rep.push_back(replacement);
+    }
+    return true;
+}
 
 bool OptionContainer::readbmlfile(const char* filename) {
     bool result = banned_mimetype_list.readItemList(filename, false, 0);
@@ -720,6 +805,7 @@ int OptionContainer::inBannedURLList(String url) {
     url.removeWhiteSpace();  // just in case of weird browser crap
     url.toLower();
     url.removePTP();  // chop off the ht(f)tp(s)://
+    url.removeMultiSlash();  // remove mulitple forward slashes
     if (url.endsWith("/")) {
         url.chop();  // chop off trailing / if any
     }
@@ -760,11 +846,10 @@ int OptionContainer::inBannedRegExpURLList(String url) {
     if (url.endsWith("/")) {
         url.chop();  // chop off trailing / if any
     }
-    RegExp re;
-    int i;
-    for (i = 0; i < banned_regexpurl_list.getListLength(); i++) {
-        re.match(banned_regexpurl_list.getItemAt(i).c_str(), url.toCharArray());
-        if (re.matched()) {
+    unsigned int i;
+    for (i = 0; i < banned_regexpurl_list_comp.size(); i++) {
+        banned_regexpurl_list_comp[i].match(url.toCharArray());
+        if (banned_regexpurl_list_comp[i].matched()) {
             return i;
         }
     }
@@ -812,8 +897,9 @@ std::deque<String> OptionContainer::ipToHostname(String ip) {
 }
 
 bool OptionContainer::isIPHostname(String url) {
-    RegExp re;
-    if (!re.match(".*[a-z|A-Z].*", url.toCharArray())) {
+//    RegExp re;
+//    re.comp(".*[a-z|A-Z].*");
+    if (!isiphost.match(url.toCharArray())) {
         return true;
     }
     return false;
@@ -886,3 +972,30 @@ bool OptionContainer::realitycheck(String s, int minl, int maxl, char* emessage)
     return true;
 }
 
+
+bool OptionContainer::precompileregexps() {
+    if (!pics1.comp("pics-label\"[ \t]*content=[\'\"]([^>]*)[\'\"]")) {
+        if (!isDaemonised) {
+            std::cerr << "Error compiling RegExp pics1." << std::endl;
+        }
+        syslog(LOG_ERR, "%s", "Error compiling RegExp pics1.");
+        return false;
+    }
+    if (!pics2.comp("[r|{ratings}] *\\(([^\\)]*)\\)")) {
+        if (!isDaemonised) {
+            std::cerr << "Error compiling RegExp pics2." << std::endl;
+        }
+        syslog(LOG_ERR, "%s", "Error compiling RegExp pics2.");
+        return false;
+    }
+    if (!isiphost.comp(".*[a-z|A-Z].*")) {
+        if (!isDaemonised) {
+            std::cerr << "Error compiling RegExp isiphost." << std::endl;
+        }
+        syslog(LOG_ERR, "%s", "Error compiling RegExp isiphost.");
+        return false;
+    }
+
+    return true;
+}
+
